*****************************************************************************
* Modulname:	SOKOBAN
* Modultype:	Library main routines
* Dest.Comp.:	HP48
* Language:	System RPL
* Author:	Detlef Mueller, Bellerbek 33, 22559 Hamburg, Germany
* Interface:	SOBA
* Description:
* To do:	- Speed up GROB+#
*
* Copyright :
*	(c) 1991-98 by Detlef Mueller. ALL rights reserved.
*
* $Id: SOBA.S 1.9 1998/10/16 22:24:28 Detlef Rel $
*
* $Log: SOBA.S $
* Revision 1.9  1998/10/16 22:24:28  Detlef
* Updated Fine logo (now shows SOKOBAN in the backround...), remed ClrField
*
* Revision 1.8  1998/10/16 21:44:57  Detlef
* Made Level@ more bullet-proof, extended RstLvl to handle the extended
* level data -- now pre-filled boxes are possible. They are banged into
* HARDBUFF after the level is decompressed & drawn.
*
* Revision 1.7  1996/11/01 23:03:45  Detlef
* Moved inclusion of level data to the end, added defines for level incl.
*
* Revision 1.6  1996/11/01 22:28:19  Detlef
* Changed to new build-sheme
*
* Revision 1.5  1996/09/30 22:57:04  Detlef
* Undo needed a check if the games is done. Fixed.
*
* Revision 1.4  1996/08/18 23:48:24  Detlef
* This is rev 1.402
*
* Edition History :
*     0.000 16.09.1991	DM	alpha version
*	001 19.09.1991	DM	add PgDt, change error handling
*	002 20.09.1991	DM	rem. Setup, Leave, Inst, add +
*     1.000 22.09.1991	DM	rem. Step, add TICK, move +,
*				add ' ID, Release, add lemming
*     1.100 25.09.1991	DM	fix levels 15,20,29,48,49,50
*	101 30.09.1991	DM	minor changes
*     1.200 08.10.1991	DM	modify repeater
*	201 12.10.1991	DM	add SetLev, add -, modify +,
*				add #=PosArry, use #=POSCOMP
*	202 22.10.1991	DM	add undo feature, XorAct, Undo, MovRem
*	203 31.10.1991	DM	correct level 36
*     1.300 02.11.1991	DM	change external data sheme,
*				rem PgDt/PGLvl, mod Level@/Level!,
*				correct level 16
*	301 11.11.1991	DM	minor changes
*	302 22.12.1991	DM	fix ERRTRAP
*	303 23.01.1992	DM	Dumdidum
*     1.400 11.02.1993	DM	fix minor bug in ShowLvl, change gc.data to
*				GCpar etc.
*	401 14.02.1993	DM	Change name to SOKOBAN, made splittable
*	402 23.02.1993	DM	ch. keys & handling, rem. ERRBEEPs
*****************************************************************************

    TITLE	SOKOBAN

*****************************************************************************
*		INCLUDE FILES
*****************************************************************************


    INCLUDE	soba.h

*****************************************************************************
*		DEFINES
*****************************************************************************


DEFINE	Version$	"V2.0(c)DM98"
DEFINE	Title$		"SOKOBAN"

DEFINE	@x		1GETLAM		( *Actor x* )
DEFINE	!x		1PUTLAM
DEFINE	@y		2GETLAM		( *Actor y* )
DEFINE	!y		2PUTLAM
DEFINE	@n		3GETLAM		( *Locations to fill* )
DEFINE	!n		3PUTLAM
DEFINE	@lvl		4GETLAM		( *Level* )
DEFINE	!lvl		4PUTLAM
DEFINE	@ug		5GETLAM		( *Undo grob* )
DEFINE	!ug		5PUTLAM
DEFINE	@un		6GETLAM		( *Undo n* )
DEFINE	!un		6PUTLAM
DEFINE	@ud		7GETLAM		( *Undo direction* )
DEFINE	!ud		7PUTLAM

DEFINE	TICK		TWO 400 setbeep

DEFINE	MAXLEVEL	FIFTY
DEFINE	EXTDATA		' ID SBpar


*****************************************************************************
*		OBJECTS
*****************************************************************************


*** SBcfg *******************************************************************
* Interface :	( --> )
* Description :	Attach this library on a warmstart
*****************************************************************************

ASSEMBLE
=SOBA-48cfg
RPL
::
    [#] SBROMID TOSRRP			( *Autoattach* )
;

*** xSOBA *******************************************************************
* Interface :	( --> )
* Description :	Main program SOKOBAN
* Calls :	Logo @Level Play
*****************************************************************************

ASSEMBLE
	CON(1)	8		* Tell parser 'Non algebraic'
RPL
xNAME	SOBA
::
    CK0
    ClrDA1IsStat RECLAIMDISP TURNMENUOFF
    ERRSET
    ::	ZERO SEVEN NDUPN		( *Initial ud, un, ug,* )
					( *lvl n y x, if no data* )
	' NULLLAM SWAP NDUPN {}N BIND	( *Generate temporary environment* )
	Logo Level@			( *Draw logo, set actual level* )
	Play
	ABND 				( *Abandon game environment* )
	TURNMENUON RECLAIMDISP
    ;
    ERRTRAP				( *On error ..* )
    ::	TURNMENUON RECLAIMDISP		( *Enshure correct display* )
	ERRJMP
    ;
;

*** Play ********************************************************************
* Interface :	( --> )
* Description :	Play the game
* Calls :	Up Left Down Right Level! Level@ RstLvl Fine
* Modifies :	n lvl
*****************************************************************************

NULLNAME	Play
::
    BEGIN
	GARBAGE				( *Force garbage collection* )
	WaitForKey
	::  kpNoShift #=casedrop	( *Unshifted key* )
	    ::	kcUpArrow OVER#=case
		::  Repeater Up
		;
		kcLeftArrow OVER#=case
		::  Repeater Left
		;
		kcDownArrow OVER#=case
		::  Repeater Down
		;
		kcRightArrow OVER#=case
		::  Repeater Right
		;

		kcSto #=casedrop Level!
		kcDelete #=casedrop RstLvl
		kcEnter #=casedrop Undo
		kcNegate #=casedrop
		::  FIFTYSIX DUP SysITE ClrSysFlag SetSysFlag
		;

		kcPlus #=casedrop	( *Inc. level* )
		::  @lvl #1+
		    MAXLEVEL #=casedrop DoBadKey
		    SetLev
		;
		kcMinus #=casedrop	( *Dec. level* )
		::  @lvl
		    ZERO #=casedrop DoBadKey
		    #1- SetLev
		;

		kcOn OVER#=
		SWAP kcBackspace #=
		ORcase
		::  BINT255d !n		( *Signal exit* )
		;

		DoBadKey
	    ;
	    kpRightShift #=casedrop	( *Rightshifted key* )
	    ::	kcSto #=casedrop Level@	( *Restore level* )

		kcOn #=case TurnOff	( *On ON turn off :*)

		DoBadKey
	    ;
	    2DROP DoBadKey		( *Illegal keyplane, do nothing* )
	;
	@n BINT255d #=
    UNTIL
;

NULLNAME	Repeater		( kc ob --> )
::
    'RRDROP ROMPTR@ DROPDUP EVAL
    [BEGIN]				( *While key down ..* )
	2DUPSWAP REPKEY?
    [WHILE]
	EVAL				( *Execute ob* )
    [REPEAT]
    3DROP

    @n #0=case Fine			( *Finish level -> yup, fine ..* )
;

*** Undo ********************************************************************
* Interface :	( --> )
* Description :	Undo last step
* Uses :	ug un
* Modify :	ud x y n screen
*****************************************************************************

NULLNAME	Undo
::
    @ud DUP#0=csDROP			( *Undo data not valid* )

    ZERO !ud				( *Mark undo data as invalid* )
    @n @un #- !n			( *Correct n* )
    @ug HARDBUFF @x @y 5ROLL		( --> grob hb x y # )
    ::					( *Undo ...* )
	ONE #=casedrop			( *right* )
	::  OVER #4- !x
	;
	TWO #=casedrop			( *left* )
	::  OVER #4+ !x
	    SWAP #4- SWAP
	;
	THREE #=casedrop		( *up* )
	::  DUP #4+ !y
	    #4-
	;
	DROP DUP #4- !y			( *down* )
    ;
    GROB! XorAct

    @n #0=case Fine			( *Finish level -> yup, fine ..* )
;

*** Up **********************************************************************
* Interface :	( --> )
* Description :	Move actor up one step
* Uses :	x
* Modify :	y n screen
*****************************************************************************

NULLNAME	Up
::
    ARRY
    [	#5AC2 #7CDB #6D79 #2ABE #1D05 #EF7A #C963
	#D8C1 #9F06 #A8BD #7956 #FB55 #4EED #CCEE
    ]
    HARDBUFF
    @x @y EIGHT #- @x #4+ @y		( --> [] hb #x #y-8 #x+4 #y )
    SUBGROB DUP				( --> [] grob grob )
    OCRC SWAPDROP HXS>#			( --> [] grob #crc )
    ROTSWAP				( --> grob [] #crc )
    #=PosArry DUP#0= case2DROP

    SWAP !ug ZERO !un THREE !ud
    @y #4- !y

    FALSE HARDBUFF			( --> #pos flg grob )
    3PICK ELEVEN #< case
    ::	ASSEMBLE
	MKSGROB 4,8,2070200020702000
	RPL
	@x @y GROB+# 2DROP TICK
    ;
    ASSEMBLE
	MKSGROB 4,12,705070005020500020702000
    RPL
    @x @y #4- MovRem
;

*** Left ********************************************************************
* Interface :	( --> )
* Description :	Move actor left one step
* Uses :	y
* Modify :	x n screen
*****************************************************************************

NULLNAME	Left
::
    ARRY
    [	#2487 #E848 #913F #0713 #B2AB #27BC #EB73
	#9204 #0428 #B190 #0DE5 #BB66 #B85D #0EDE
    ]
    HARDBUFF
    @x EIGHT #- @y @x @y #4+		( --> hb #x-8 #y #x #y+4 )
    SUBGROB DUP
    OCRC SWAPDROP HXS>#
    ROTSWAP
    #=PosArry DUP#0= case2DROP

    SWAP !ug ZERO !un TWO !ud
    @x #4- !x

    FALSE HARDBUFF			( --> #pos flg grob )
    3PICK ELEVEN #< case
    ::	ASSEMBLE
	MKSGROB 8,4,22772200
	RPL
	@x @y GROB+# 2DROP TICK
    ;
    ASSEMBLE
	MKSGROB 12,4,7520527075200000
    RPL
    @x #4- @y MovRem
;

*** Down ********************************************************************
* Interface :	( --> )
* Description :	Move actor down one step
* Uses :	x
* Modify :	y n screen
*****************************************************************************

NULLNAME	Down
::
    ARRY
    [	#5AC2 #960D #EF7A #7956 #CCEE #6D79 #A1B6
	#D8C1 #4EED #FB55 #2ABE #A8BD #9F06 #1D05
    ]
    HARDBUFF
    @x @y #4+ @x #4+ @y TWELVE #+ 	( --> hb #x #y+4 #x+4 #y+12 )
    SUBGROB DUP
    OCRC SWAPDROP HXS>#
    ROTSWAP
    #=PosArry DUP#0= case2DROP

    SWAP !ug ZERO !un FOUR !ud
    @y #4+ !y

    FALSE HARDBUFF			( --> #pos flg grob )
    3PICK ELEVEN #< case
    ::	ASSEMBLE
	MKSGROB 4,8,2070200020702000
	RPL
	@x @y #4- GROB+# 2DROP TICK
    ;
    ASSEMBLE
	MKSGROB 4,12,207020005020500070507000
    RPL
    @x @y #4- MovRem
;

*** Right *******************************************************************
* Interface :	( --> )
* Description :	Move actor right one step
* Uses :	y
* Modify :	x n screen
*****************************************************************************

NULLNAME	Right
::
    ARRY
    [	#2487 #88BB #27BC #0DE5 #0EDE #913F #3D03
	#9204 #B85D #BB66 #0713 #B190 #0428 #B2AB
    ]
    HARDBUFF
    @x #4+ @y @x TWELVE #+ @y #4+	 ( --> hb #x+4 #y #x+12 #y+4 )
    SUBGROB DUP
    OCRC SWAPDROP HXS>#
    ROTSWAP
    #=PosArry DUP#0= case2DROP

    SWAP !ug ZERO !un ONE !ud
    @x #4+ !x

    FALSE HARDBUFF			( --> #pos flg grob )
    3PICK ELEVEN #< case
    ::	ASSEMBLE
	MKSGROB 8,4,22772200
	RPL
	@x #4- @y GROB+# 2DROP TICK
    ;
    ASSEMBLE
	MKSGROB 12,4,2570725025700000
    RPL
    @x #4- @y MovRem
;

*** MovRem ******************************************************************
* Interface :	( #pos gr hb x y --> )
* Description :	Remainder of Up, Left, Down, Right
* Uses :	lvl x y n screen
* Modifies :	n un screen
*****************************************************************************

NULLNAME	MovRem
::
    RDROP
    GROB+# DROP				( --> #pos )
    TICK

    THIRTEEN #=casedrop
    ::	@n #1- !n
	MINUSONE !un
    ;

    FOURTEEN #<> ?SEMI
    @n #1+ !n
    ONE !un
;

*** Level! ******************************************************************
* Interface :	( --> )
* Description :	Store a snapshot from the game into EXTDATA
* Uses :	lvl x y n screen
* Modifies :	EXTDATA
*****************************************************************************

NULLNAME	Level!
::
    [#] SBROMID @x @y @n @lvl
    HARDBUFF ZEROZERO 76 SIXTYFOUR SUBGROB
    SIX {}N DUP OSIZE #5-		( #id #x #y #n #lvl grob --> {} #sz )
    CODE				( {} #sz --> LibDat )
* CPU			A	C	D1
	GOSBVL	=POP#	sz		->stk
	C=A	A		sz
	RSTK=C		** Save size
	A=DAT1	A	->{}
	AD1EX		->stk		->{}
	LC(5)	=DOEXT0		LD
	DAT1=C	A	** {}prlg becomes LDprlg
	D1=D1+	5			->#prlg
	C=RSTK			sz
	DAT1=C	A	** #prlg becomes length
	D1=A				->stk
	GOVLNG	=Loop
    ENDCODE

    EXTDATA SAFESTO
;

*** Level@ ******************************************************************
* Interface :	( --> )
* Description :	If EXTDATA is present in the path, setup the game environment
*		with the data stored in it. Else call RstLvl and Level!.
* Uses :	EXTDATA
* Modifies :	lvl x y n screen
* Calls :	RstLvl Level!
* Ed. History :
*****************************************************************************

NULLNAME	Level@
::
    EXTDATA SAFE@ NOTcase SetLvl1	( *No data to restore from* )

    DUPTYPEREAL? case			( *Support level select* )
    ::	COERCE #1- MAXLEVEL OVER#>
	case SetLev
	DROP SetLvl1
    ;
					( *Library Data - maybe...* )
    TOTEMPOB

    CODE				( ob --> ob f/t )
*			A	C	D1
	GOSBVL	=SAVPTR

	C=DAT1	A		->ld	->stk
	D1=C				->ld

	A=DAT1	A	prlg

	LC(5)	=DOEXT0		LDprlg
	?C#A	A
	GOYES	gpptfl				** ob not ld
*						** ob is ld, check lid
	D1=D1+	10			->lid
	A=DAT1	A	lid

	LC(5)	SBROMID		LID
	?C#A	A
	GOYES	gpptfl				** not our ld
*						** convert ld -> {}
	D1=D1-	5			->len
	LC(5)	=DOBINT 	#prlg
	DAT1=C	A	** len becomes #prlg

	D1=D1-	5			->prlg
	LC(5)	=DOLIST 	{}prlg
	DAT1=C	A	** LDprlg becomes {}prlg
*						** CC
gpptfl	GOVLNG	=GPPushT/FLp

    ENDCODE

    casedrop SetLvl1

    INCOMPDROP				( {} --> #id #x #y #n #lvl grob )
    ZEROZERO ROT XYGROBDISP
    !lvl !n !y !x DROP
    ZERO !ud				( *Undo data not valid* )
    ShowLvl				( *Display level no.* )
;


NULLNAME	SetLvl1
::
    RstLvl				( *Extract level 1* )
    Level!				( *Generate data in RAM* )
;


NULLNAME	ShowLvl		( --> *Display level no.* )
::
    @lvl #1+ #>$
    DUPLEN$ #1= IT
    ::	CHR_Space >H$
    ;
    $>grob HARDBUFF
    103 THIRTYSIX GROB!
;

*** RstLvl ******************************************************************
* Interface :	( --> )
* Description :	Prepare screen and tmp.vars for level n
* Uses :	lvl
* Modifies :	x y n screen
* Calls :	ShowLvl LvlPic LvlData Psh#ALp
*****************************************************************************

NULLNAME	RstLvl
::
    ShowLvl
    LvlPic				( --> $ )
    @lvl 76 #* #1+DUP 75 #+		( $ --> $ #lvl*76+1 #lvl*76+76 )
    SUB$				( *Get compressed level image* )

    ZEROZERO !x !y			( x = y = 0, --> $ )
					( *Clear game area* )
    HARDBUFF ZEROZERO 76 SIXTYFOUR GROB!ZERODRP

    GARBAGE				( *Force garbage collection* )

    BEGIN
	DUP @y NINETEEN #* @x #+

    CODE				( #16/ )
*				A
	GOSBVL	=POP#		#
	GOSBVL	=SAVPTR
	ASR	A		#/16
	GOVLNG	=PUSH#ALOOP
    ENDCODE

	#1+ SUB$1#			( ... --> $ #[y*19+x]/16+1 )

	FOUR ZERO_DO (DO)
	    DUP THREE #AND		( $ # --> $ # #' *Get lower two bits* )
	    DUP#0= ITE_DROP
	    ::  {   GROB 12 400004000050A050A0	( *Border* )
		    GROB 12 400004000070507000	( *Box* )
		    GROB 12 400004000000200000	( *Dot* )
		}
		SWAP NTHCOMPDROP	( $ # #' {} --> $ # grob )
		HARDBUFF @x @y GROB!
	    ;

	    #2/ #2/			( $ # --> $ #' *Shift out lower 2 bits* )
	    @x #4+			( $ # --> $ # # )
	    72 OVER#< IT
	    ::	DROPZERO
		@y #4+ !y
	    ;

	    !x
	LOOP
	DROP
	@y SIXTY #>
    UNTIL
    DROP

    LvlData
    @lvl FOUR #* #1+DUP #3+		( --> {} #lvl*4+1 #lvl*4+4 )
    SUBCOMP INCOMPDROP			( ... --> #tofill #x #y {} )

    DUPNULL{}?
    ITE_DROP
    ::	INNERCOMP #2/
	ZERO_DO (DO)
	    GROB 12 400004000070707000	( *Filled box* )
	    HARDBUFF 2SWAP		( --> ... hb grob #x #y )
	    GROB!
	LOOP
    ;

    !y !x !n
    ZERO !ud				( *Undo data not valid* )
    XorAct				( *XOR actor at x, y* )
;


NULLNAME	XorAct			( --> *XOR actor at x, y* )
::
    FALSE HARDBUFF
    GROB 12 400004000020702000
    @x @y GROB+# DROP
;

*** Logo ********************************************************************
* Interface :	( --> )
* Description :	Draw the SOKOBAN logo
* Uses :	Version$ Title$
* Modifies :	screen
*****************************************************************************

NULLNAME	Logo
::
    HARDBUFF 83 ONE 128 TWENTY
    GROB!ZERODRP
    86 FIFTYSIX 128 OVER
    LINEOFF

    86 FIFTYSEVEN Version$ $>grob
    83 TWENTY
    ASSEMBLE
	MKGROB 47,36
	NIBHEX	FFFFFFFFFFF7555555555555BAAAAAAAAAA655FFFFFF7555BA100000EAA655
	NIBHEX	100000A555BA1000002BA6551000002655BA1EFFF12CA6551600A12455BA1A
	NIBHEX	00512CA6551218A12455BA1220512CA65512CFF12455BA1240012CA6551240
	NIBHEX	012455BA1240012CA6551240012455BA1240012CA6551240012455BA124001
	NIBHEX	2CA6551240012455BA1240012CA65512CFF12455BA1245122CA65512CA0424
	NIBHEX	55BA1245082CA65512C0003455BA12CFFF3CA6553200000455BA6200000CA6
	NIBHEX	55D200000455BAA300000CA6555FFFFFF755BAAAAAAAAAA6FFFFFFFFFFF7
    RPL
    85 SIX Title$ $>GROB

    XYGROBDISP XYGROBDISP XYGROBDISP

    83 ONE 128 OVER
    OVER TWO OVER SEVENTEEN
    83 OVER 127 OVER
    83 TWO 83 SIXTEEN
    84 EIGHTEEN 129 EIGHTEEN
    OVER TWO OVER SEVENTEEN

    SIX ZERO_DO (DO)
	LINEON
    LOOP
;

*** Fine ********************************************************************
* Interface :	( --> )
* Description :	Draw the fine logo et c.
* Modifies :	lvl n screen
* Calls :	Logo RstLvl Level!
*****************************************************************************

NULLNAME	Fine
::
    ASSEMBLE
	MKGROB 38,55
	NIBHEX	00F10000000802000000040200000004040000003604EC3E404504154150C7
	NIBHEX	041541D02C061D3F5128C91EF15218080103501010901E5010186800302817
	NIBHEX	4880402820484080244048508070C1E0708270AF7BF043801BA60FF0F1050C
	NIBHEX	66015301080102A28088A00257408EF102A9308702815150C30041A1B0A300
	NIBHEX	A2515E520061A1100600E25110080051A1210000D25120000841A1210008C2
	NIBHEX	5140000441A1600002C251A0008341A12100F2C251270C3241A1220022C251
	NIBHEX	24002241A1240832C25124082241A1240022C25124002241A1240022C25124
	NIBHEX	004241A12800C2C2512C004341A12C00C3C25328004041A6280040C25D2800
	NIBHEX	4041AA380080C255FF008F71AAAA100BA2FFFF108FF30000FF7000
    RPL
    HARDBUFF 87 TWO GROB!

    300 554 OVER 659 75 740 150 6PICK 4PICK 6PICK 4PICK OVER
    SIX ZERO_DO (DO)
	setbeep
    LOOP

    VERYVERYSLOW

    @lvl #1+
    MAXLEVEL #=casedrop
    ::	BINT255d !n
	HARDBUFF ZEROZERO 76 SIXTYFOUR GROB!ZERODRP
	ONE TWENTYTHREE "You did it !" $>GROB XYGROBDISP
	ONE THIRTYONE "Bye, bye \1F" $>GROB XYGROBDISP
	ZERO
	BEGIN
	    #1+
	    {   ASSEMBLE
		MKSGROB 6,10,00C3C18381818181C181
		MKSGROB 7,10,82C1C18381C1C585C2C0
		MKSGROB 7,10,0041C18183C1C1E3C366
		MKSGROB 6,10,0081C3C38181C181E323
		MKSGROB 6,10,00C3C1C381818181C181
		MKSGROB 7,10,82C1C18381818585C2C0
		MKSGROB 7,10,0041C18183818183C366
		MKSGROB 6,10,0081C3C381818181E323
		RPL
	    }
	    OVER SEVEN #AND #1+ NTHCOMPDROP
	    HARDBUFF 3PICK FIFTYFOUR
	    GROB!
	    DUP THREE #AND #2= IT
	    ::  THREE 400 setbeep
	    ;
	    %.1 dowait
	    ATTN? IT
	    ::  DROP SEVENTYFOUR
		ATTNFLGCLR
	    ;
	    SEVENTYFOUR OVER#=
	UNTIL DROP
	FLUSHKEYS
    ;

    Logo SetLev
;


NULLNAME	SetLev			( #level --> *Set level* )
::
    !lvl SetLvl1
;

** #=PosArry ****************************************************************
* Interface :	( arry # --> #pos/#0 )
* Description :	Search in <arry> of BINT's for <#>.
* Defines :	Psh#ALp
*****************************************************************************

NULLNAME #=PosArry
CODE
* CPU			A	B	C	D0	D1
	GOSBVL	=POP#	n
	D1=D1+  5		update D
	D=D+1   A		update M
	GOSBVL	=SAVPTR
	D1=D1-	5					&&arry
	C=DAT1	A			&arry
	D0=C					&arry
	D1=C						&arry
	D0=D0+	5				&len
	C=DAT0	A			len
	B=C	A		len
	CD1EX				&arry
	C=C+B	A			&lastelem
	D1=C						&lastelem
	D0=D0+	15				&dim1len
	C=DAT0	A			dim1len
	B=A	A		n
	A=C	A	pos
	GOTO	PAEntry

PALoop	C=DAT1	A			x
	?C=B	A
	GOYES	PAFound
	D1=D1-	5					&prev.elem
PAEntry	A=A-1	A	--pos
	GONC	PALoop

PAFound	A=A+1	A	++pos
Psh#ALp	GOVLNG	=PUSH#ALOOP

ENDCODE

*****************************************************************************
*		LEVEL DATA
*****************************************************************************

ASSEMBLE

L01-10	EQU	1
L11-20	EQU	1
L21-30	EQU	1
L31-40	EQU	1
L41-50	EQU	1

RPL

    INCLUDE	sobalvl.inc

******************************************************************************
*			End of SOBA.S
******************************************************************************
